# -*- coding: iso_8859_1 -*-

#------------------------------------------------------------------------------------------------------------
#		Calls to Win32 API functions to manage windows positions, size, etc
#------------------------------------------------------------------------------------------------------------

from ctypes import *
import win32con


#----------------------------------------------------------------------------------------
#			Actions on the windows, functions without arguments
#----------------------------------------------------------------------------------------

def set_focus(title):
	"""
	---------------------
	Purpose
	Sets the focus on the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window that will get the focus.
	---------------------
	Output = no output.
	---------------------
	"""
	
	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.SetForegroundWindow(h)
	#-------------------------------------

def maximize_window(title):
	"""
	---------------------
	Purpose
	Maximizes the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window that will be maximized.
	---------------------
	Output = no output.
	---------------------
	"""

	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.ShowWindow(h, win32con.SW_MAXIMIZE)
	#-------------------------------------

def scroll_up_window(title):
	"""
	---------------------
	Purpose
	Scrolls up the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window that will be scrolled up.
	---------------------
	Output = no output.
	---------------------
	"""

	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.SetScrollPos(h, win32con.SB_VERT, 0, 1)
	windll.User32.SendMessageA(h, win32con.WM_VSCROLL, win32con.SB_TOP, 0)
	#-------------------------------------

def close_window(title):
	"""
	---------------------
	Purpose
	Closes the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window that will be closed.
	---------------------
	Output = no output.
	---------------------
	"""

	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.SendMessageA(h, win32con.WM_CLOSE, 0, 0)
	#-------------------------------------
	
#----------------------------------------------------------------------------------------
#			Get information on the windows
#----------------------------------------------------------------------------------------

def get_window_size(title):
	"""
	---------------------
	Purpose
	Gets the size of the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window for which the size is returned.
	---------------------
	Output ([integer, integer]) = two elements list, giving the window size respectively in the horizontal and vertical directions.
	---------------------
	"""
	
	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.user32.GetWindowRect(h, byref(s))
	return [s[2]-s[0], s[3]-s[1]]

def get_window_position(title):
	"""
	---------------------
	Purpose
	Gets the position of the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window for which the position is returned.
	---------------------
	Output ([integer, integer]) = two elements list, giving the window position respectively in the horizontal (measured from the left screen limit) and vertical (measured from the top screen limit) directions.
	---------------------
	"""

	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.user32.GetWindowRect(h, byref(s))
	return [s[0], s[1]]		#position of top left corner
	
#----------------------------------------------------------------------------------------
#			Actions on the windows, functions with arguments
#----------------------------------------------------------------------------------------

def move_window(title, xy):
	"""
	---------------------
	Purpose
	Moves the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window for which the position is returned.
	* xy ([integer, integer]) = two elements list, giving the new window position, respectively in the horizontal (measured from the left screen limit) and vertical (measured from the top screen limit) directions.
	---------------------
	Output = no output.
	---------------------
	"""
	
	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.User32.GetWindowRect(h, byref(s))
	windll.User32.MoveWindow(h, xy[0], xy[1], s[2]-s[0], s[3]-s[1], 1)
	#-------------------------------------

def resize_window(title, xy):
	"""
	---------------------
	Purpose
	Resizes the window that has the specified title.
	This function is based on the Windows API.
	---------------------
	Inputs
	* title (string) = title of the window for which the position is returned.
	* xy ([integer, integer]) = two elements list, giving the new window size, respectively in the horizontal and vertical directions.
	---------------------
	Output = no output.
	---------------------
	"""

	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.User32.GetWindowRect(h, byref(s))
	windll.User32.MoveWindow(h, s[0], s[1], xy[0], xy[1], 1)
	#-------------------------------------

#----------------------------------------------------------------------------------------
#			Get general information
#----------------------------------------------------------------------------------------

def get_screen_size():
	"""
	---------------------
	Purpose
	Retrieves the screen size in pixels in both directions.
	This function is based on the Windows API.
	---------------------
	Inputs = no inputs
	---------------------
	Output ([integer, integer]) = two elements list, giving the screen size in pixels, respectively in the horizontal and vertical directions.
	---------------------
	"""
	
	x = windll.User32.GetSystemMetrics(win32con.SM_CXSCREEN)
	y = windll.User32.GetSystemMetrics(win32con.SM_CYSCREEN)
	return [x,y]

