# -*- coding: iso_8859_1 -*-

import numpy as np
import sys

def project_stars(alpha, delta, alpha0, delta0, direction):
	"""
	---------------------
	Purpose
	Transforming a list of (alpha, delta) star coordinates into (x, y) cartesian coordinates using a stereographic projection, which is conformal (=does not change the shapes).
	The transformation is similar as the one used in:
	N.Kaiser, G.Wilson, G.Luppino, H.Dahle "A photometric study of the supercluster MS0302 with the UH8K CCD camera - image processing and object catalogs"
	arXiv:astro-ph/9907229v1 (16 Jul 1999)
	---------------------
	Inputs
	* stars (Numpy array) = star catalog, a Numpy array with at stars[:,0] = alpha and stars[:,1] = delta coordinates in degrees.
	* center ([float, float]) = center of the field of view to be transformed, provided as [alpha, delta] coordinates in degrees.
	The specified point is the point of the projection plane that is tangent to the unit sphere.
	* direction (string) = defines the azimuth direction of the vertical axis rv. Can be 'North' or 'South' depending on whether rv is directed towards North or South pole. 
	---------------------
	Output (Numpy array) = Numpy array such that: Output[:,0] = vertical projected coordinate, Output[:,1] = horizontal projected coordinate..
	---------------------
	"""
	if direction == 'North':
		flag = 1.
	elif direction == 'South':
		flag = -1.
	else:
		sys.exit("direction not recognised, function astrometry.project_stars() aborted")
		
	alpha0 = alpha0*np.pi/180.
	delta0 = delta0*np.pi/180.

	axis = [np.cos(delta0)*np.cos(alpha0), np.cos(delta0)*np.sin(alpha0), np.sin(delta0)]
	rvert = [-flag*np.sin(delta0)*np.cos(alpha0), -flag*np.sin(delta0)*np.sin(alpha0), flag*np.cos(delta0)]
	rhoriz = [flag*np.sin(alpha0), -flag*np.cos(alpha0), 0.0]
	
	stars_X = np.cos(delta*np.pi/180.)*np.cos(alpha*np.pi/180.)
	stars_Y = np.cos(delta*np.pi/180.)*np.sin(alpha*np.pi/180.)
	stars_Z = np.sin(delta*np.pi/180.)
	
	dot_axis = stars_X*axis[0] + stars_Y*axis[1] + stars_Z*axis[2]
	dot_rvert = stars_X*rvert[0] + stars_Y*rvert[1] + stars_Z*rvert[2]
	dot_rhoriz = stars_X*rhoriz[0] + stars_Y*rhoriz[1] + stars_Z*rhoriz[2]
	
	return np.vstack([1.e6*dot_rvert / dot_axis, 1.e6*dot_rhoriz / dot_axis]).transpose()

# def append_field(rec, name, arr, dtype=None):
    # arr = np.asarray(arr)
    # if dtype is None:
        # dtype = arr.dtype
    # newdtype = np.dtype(rec.dtype.descr + [(name, dtype)])
    # newrec = np.empty(rec.shape, dtype=newdtype)
    # for field in rec.dtype.fields:
        # newrec[field] = rec[field]
    # newrec[name] = arr
    # return newrec
