# -*- coding: iso_8859_1 -*-

import urllib

def send_query_vizier(catalog, position, conesize, constraint, filename = 'vizier.xml'):
	"""
	---------------------
	Purpose
	Sends a query to Vizier and retrieve a list of stars with astrometric (alpha, delta, etc) or photometric (magnitudes, etc) information
	The data is saved in a specific xml file, with VOtable format
	The syntax to send a Vizier query is described there: http://vizier.u-strasbg.fr/doc/asu-summary.htx
	---------------------
	Inputs
	* catalog (string) = name of the catalog, following Vizier syntax
	Popular Vizier catalogs include:
	'II/294' - Sloan SDSS photometric catalog Release 7 (2009)
	'2MASS-PSC' - 2MASS point source catalog (2003)
	'GSC2.3' - Version 2.3.2 of the HST Guide Star Catalog (2006)
	'USNO-B1' - Verson B1 of the US Naval Observatory catalog (2003)
	'NVSS'  - NRAO VLA Sky Survey (1998)
	'B/DENIS/DENIS' - 2nd Deep Near Infrared Survey of southern Sky
	'I/259/TYC2' - Tycho-2 main catalog (2000)
	'I/311/HIP2' - Hipparcos main catalog, new reduction (2007)	Each string gives the complete filename (including path) of a file that contains the lightcurve measurements.
	* position (string or [float,float]) = position name (resolved by Sesame) or alpha/delta position (in degrees) giving the center of the star field to be requested
	* conesize (None, float or [float,float]) = size in arcmin, corresponds to the radius (float) or alpha/delta box rectangular size ([float,float]) defining the star field requested
	If equal to None (default), a radius of 5 arcmin is used
	* constraint (string) = list of constraints to be applied on the catalogs fields to filter the retrieved stars (Vizier syntax)
	For instance: 'B1mag<=13'
	Multiple contraints are separated by comma: 'B1mag<13,B2mag<14'
	* filename (string) = optional parameter, default is 'vizier.xml'.
	Specifies the xml file in which the data from the Vizier request is stored.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	---------------------
	Output = no output
	---------------------
	"""
	#------ (1) Preparing url for Vizier ----------------------

	cat = "?-source=" + catalog
	if isinstance(position,str):
		obj = "&-c=" + position
	else:
		obj = "&-c.ra=" + str(position[0]) + "&-c.dec=" + str(position[1])
	if conesize==None:
		dis = "&-c.rm=05"
	elif isinstance(conesize,list):
		dis = "&-c.bm=" + str(int(conesize[0])) + "/" + str(int(conesize[1]))
	else:
		dis = "&-c.rm=" + str(int(conesize))
	if len(constraint)==0:
		cons = ""
	else:
		constraint = constraint.replace(" ","")
		constraint = constraint.replace(",","&")
		constraint = constraint.replace("<","=%3C")
		constraint = constraint.replace(">","=%3E")
		constraint = constraint.replace("+","%2B")
		constraint = constraint.replace("/","%2F")
		constraint = constraint.replace("!","=!")
		cons = "&" + constraint
		
	url = "http://webviz.u-strasbg.fr/viz-bin/votable/" + cat + obj + dis + cons + "&-out.max=unlimited"

	#------ (2) Sending query to Vizier ----------------------

	print
	print "--- retrieving data from Vizier ---"
	
	fp = urllib.urlopen(url)
	op = open(filename, "wb")

	while 1:
		s = fp.read(8192)
		if not s:
			break
		op.write(s)

	fp.close()
	op.close()
	#----------- end of function -----------------------------

def send_query_skybot(date, position, conesize, obscode='500', outputlevel=1, filename = 'skybot.xml'):
	"""
	---------------------
	Purpose
	Sends a query to SkyBoT and retrieve a list of solar system objects with astrometric (alpha, delta, etc) and photometric (magnitudes, etc) information
	The data is saved in a specific xml file, with VOtable format
	The syntax to send a SkyBoT query is described there: http://vo.imcce.fr/webservices/skybot/?conesearch
	---------------------
	Inputs
	* date (float) = Julian date at which information is requested
	* position ([float,float]) = alpha/delta position (in degrees) giving the center of the sky area for which the request is formulated
	* conesize (None, float or [float,float]) = size in arcmin, corresponds to the radius (float) or alpha/delta box rectangular size ([float,float]) defining the sky area for which the request is formulated
	If equal to None (default), a radius of 5 arcmin is used
	* obscode (string) = optional parameter, default is '500'. Corresponds to the IAU observatory code for which information is requested. '500' corresponds to Earth center (geocentric positions).
	* outputlevel (integer) = optional parameter, default is 1.
	Defines the level of detail requested (see the SkyBoT webpage for more information).
	- 1 corresponds to 'object'
	- 2 corresponds to 'basic'
	- 3 corresponds to 'obs'
	- 4 corresponds to 'all'
	* filename (string) = optional parameter, default is 'skybot.xml'.
	Specifies the xml file in which the data from the SkyBoT request is stored.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	---------------------
	Output = no output
	---------------------
	"""

	#------ (1) Preparing url for SkyBoT ----------------------

	dat = "-ep=" + str(date)
	pos = "&-ra=" + str(position[0]) + "&-dec=" + str(position[1])
	if conesize==None:
		cone = "&-rm=05"
	elif isinstance(conesize,list):
		cone = "&-bm=" + str(int(conesize[0])) + "x" + str(int(conesize[1]))
	else:
		cone = "&-rm=" + str(int(conesize))
	if outputlevel ==1:
		out = "&-output=object"
	elif outputlevel ==2:
		out = "&-output=basic"
	elif outputlevel ==3:
		out = "&-output=obs"
	else:
		out = "&-output=all"
	loc = "&-loc=" + obscode

	url = "http://vo.imcce.fr/webservices/skybot/skybotconesearch_query.php?" + dat + pos + cone + "&-mime=votable" + out + loc + "&-from=AsPyLib"

	#------ (2) Sending query to SkyBoT ----------------------

	print
	print "--- retrieving data from SkyBoT ---"
			
	fp = urllib.urlopen(url)
	op = open(filename, "wb")

	while 1:
		s = fp.read(8192)
		if not s:
			break
		op.write(s)

	fp.close()
	op.close()
	#----------- end of function -----------------------------
